﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Interfaces;
using ff14bot.NeoProfiles;
using System.Xml;
using ff14bot.Helpers;
using ff14bot.Settings;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace CraftBuddy
{
    public class CraftBuddy : IBotPlugin
    {
        internal List<Recipe> Recipes { get; set; }
        internal List<ProfileTemplate> ProfileTemplates { get; private set; }
        internal string RecipesPath { get; private set; }
        internal string TemplatesPath { get; private set; }
        internal string DefaultTemplate { get; set; }
        internal cbLogger Logger { get; private set; }
        internal cbSettings Settings { get; private set; }
        private CraftWindow _craftWindow { get; set; }

        private CraftBuddy()
        {

        }

        #region Plugin Information

        public string Author
        {
            get { return " Cannibal"; }
        }

        public string Name
        {
            get { return "CraftBuddy"; }
        }

        public string Description
        {
            get { return "Crafting profile generator"; }
        }

        public Version Version
        {
            get { return new Version(0, 0, 3); }
        }


        public bool WantButton
        {
            get { return true; }
        }

        public string ButtonText
        {
            get { return "CraftBuddy"; }
        }

        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }

        #endregion

        public void OnButtonPress()
        {
            if(_craftWindow == null || _craftWindow.IsDisposed || _craftWindow.Disposing)
            {
                // Only attempt to load recipes from file when the player's current class
                // is a valid crafting class.
                if(IsValidCraftClass(Core.Me.CurrentJob))
                {
                    Recipes = GetRecipes();
                    if(Recipes == null)
                    {
                        Logger.Write("Failed to load recipes!");
                        return;
                    }

                    _craftWindow = new CraftWindow(this);
                    _craftWindow.Show();
                }
                else
                {
                    MessageBox.Show("Current class is not a valid crafting class!", "CraftBuddy", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
            else
            {
                _craftWindow.BringToFront();
            }
        }


        public void OnInitialize()
        {
            // Create a new logger
            if(Logger == null)
                Logger = new cbLogger();

            // Get plugin path and check if recipe files, and templates exist
            GlobalSettings _settings = GlobalSettings.Instance;
            string PluginPath = Path.Combine(_settings.PluginsPath, "CraftBuddy");
            RecipesPath = Path.Combine(PluginPath, @"Data\Recipes");
            if(!Directory.Exists(RecipesPath))
            {
                Logger.Write(LoggerType.Error, string.Format("Failed to load the recipes from '{0}'", RecipesPath));
                return;
            }
            TemplatesPath = Path.Combine(PluginPath, @"Data\Templates");
            if(!Directory.Exists(TemplatesPath))
            {
                Logger.Write(LoggerType.Error, string.Format("Template directory not found at '{0}'", TemplatesPath));
                return;
            }

            // Load settings
            string SettingsPath = Path.Combine(PluginPath, @"Data\Settings\settings.json");
            if(!File.Exists(SettingsPath))
            {
                Logger.Write(LoggerType.Error, string.Format("Settings file not found at '{0}'", SettingsPath));
                return;
            }
            else
            {
                using(StreamReader sr = new StreamReader(SettingsPath))
                {
                    string jsonData = sr.ReadToEnd();
                    Settings = JsonConvert.DeserializeObject<cbSettings>(jsonData);
                }
            }


            ReloadTemplates();
        }



        public void ReloadTemplates()
        {
            // Generate ProfileTemplate objects
            ProfileTemplates = new List<ProfileTemplate>();
            foreach(string templatePath in Directory.GetFiles(TemplatesPath))
            {
                if(!templatePath.ToLower().EndsWith(".xml"))
                    continue;

                string filename = Path.GetFileNameWithoutExtension(templatePath);
                if(string.IsNullOrEmpty(filename))
                {
                    continue;
                }
                ProfileTemplate pt = new ProfileTemplate(filename, templatePath);
                ProfileTemplates.Add(pt);
            }
        }

        public void OnPulse()
        {
        }

        public void OnShutdown()
        {
        }

        public void OnDisabled()
        {
            if(_craftWindow != null && !_craftWindow.IsDisposed && !_craftWindow.Disposing)
                _craftWindow.Dispose();
        }

        public void OnEnabled()
        {
        }


        /// <summary>
        /// Returns the current Job's recipes collection filename.
        /// This is used by the GetRecipes method to get a valid source file for importing.
        /// </summary>
        /// <param name="currentJob"></param>
        /// <returns>string on success, Null on failure.</returns>
        private string GetRecipePath(ClassJobType currentJob)
        {
            // Define valid jobs and the according recipes source file path.
            Dictionary<ClassJobType, string> jobs = new Dictionary<ClassJobType, string>
            {
                {ClassJobType.Alchemist, "recipes_alchemist.txt"},
                {ClassJobType.Armorer, "recipes_armorer.txt"},
                {ClassJobType.Blacksmith, "recipes_blacksmith.txt"},
                {ClassJobType.Carpenter, "recipes_carpenter.txt"},
                {ClassJobType.Culinarian, "recipes_culinarian.txt"},
                {ClassJobType.Goldsmith, "recipes_goldsmith.txt"},
                {ClassJobType.Leatherworker, "recipes_leatherworker.txt"},
                {ClassJobType.Weaver, "recipes_weaver.txt"}
            };

            if(jobs.ContainsKey(currentJob))
            {
                return jobs[currentJob];
            }

            return null;
        }

        ///// <summary>
        ///// Returns a Recipe object when the passed Parameter contains a valid raw recipe string.
        ///// </summary>
        ///// <param name="recipeRaw"></param>
        ///// <returns></returns>
        private Recipe ParseRecipe(string recipeRaw)
        {
            if(recipeRaw.IndexOf(':') == -1)
                return null;

            string[] recipeSplit = recipeRaw.Split(':');
            if(recipeSplit.Length != 2) return null;

            uint recipeId;
            string recipeName = recipeSplit[1].Substring(1);
            if(uint.TryParse(recipeSplit[0], out recipeId))
                return new Recipe(recipeId, recipeName.Replace("-", " "));

            return null;
        }

        /// <summary>
        /// Loads recipes from file and creates a list of recipe objects.
        /// If internal directories are not found the return value might be null!
        /// </summary>
        internal List<Recipe> GetRecipes()
        {
            string jobPath = GetRecipePath(Core.Me.CurrentJob);
            if(jobPath == null)
            {
                Logger.Write(LoggerType.Error, "Failed to obtain the recipes for your current class.");
                Logger.Write(LoggerType.Error, "Make sure you have the job learned and set!");
                return null;
            }

            jobPath = string.Format("{0}\\{1}", RecipesPath, jobPath);
            if(!File.Exists(jobPath))
            {
                Logger.Write(LoggerType.Error, "Recipe Directory not found!");
                Logger.Write(LoggerType.Error, string.Format("Make sure the recipes are stored at '{0}'.", jobPath));
                return null;
            }

            // Add the lines of the read file to the list variable.
            List<string> rawRecipeList = File.ReadLines(jobPath).Where(line => line != null).ToList();

            // Create a Recipe object for each item inside the rawRecipeList and store them in the recipes list.
            List<Recipe> recipes = new List<Recipe>();
            foreach(string recipe in rawRecipeList)
            {
                Recipe recipeObj = ParseRecipe(recipe);
                if(recipeObj != null)
                    recipes.Add(recipeObj);
            }

            if(!recipes.Any()) return null;

            return recipes;
        }


        /// <summary>
        /// Returns bool when the passed job is a valid crafting job.
        /// </summary>
        /// <param name="job">ClassJobType jobt</param>
        /// <returns>True if the passed job is a crafting job, False otherwise.</returns>
        internal bool IsValidCraftClass(ClassJobType job)
        {
            bool retBool;
            switch(job)
            {
                case ClassJobType.Alchemist:
                case ClassJobType.Armorer:
                case ClassJobType.Carpenter:
                case ClassJobType.Culinarian:
                case ClassJobType.Goldsmith:
                case ClassJobType.Leatherworker:
                case ClassJobType.Weaver:
                case ClassJobType.Blacksmith:
                    retBool = true;
                    break;
                default:
                    retBool = false;
                    break;
            }

            return retBool;
        }


        internal void generateProfile(Recipe recipe, ProfileTemplate pt)
        {
            // Read the contents of the base profile.
            string profileContent;
            using(StreamReader sr = new StreamReader(pt.Path))
            {
                profileContent = sr.ReadToEnd();
            }

            // Give feedback if the profile content was empty (we can't recover from this error).
            if(string.IsNullOrEmpty(profileContent))
            {
                Logger.Write(LoggerType.Error, "Base Template was empty!");
                Logger.Write(LoggerType.Error, string.Format("Template path: {0}", pt.Path));
                return;
            }

            // Replace the recipe id in the profile with the currently selected recipe in the craftbuddy window.
            string replacedContent = Regex.Replace(profileContent, "RecipeId=\".*?\"", string.Format("RecipeId=\"{0}\"", recipe.Id));
            if(string.IsNullOrEmpty(replacedContent))
            {
                Logger.Write(LoggerType.Error, "Failed to generate new profile!");
                return;
            }

            // Write the changed content of the base profile back to file.
            using(StreamWriter sw = new StreamWriter(pt.Path))
            {
                sw.WriteLine(replacedContent.Trim());
                Logger.Write("Profile has been created!");
            }

            // TODO: Move to seperate function (maybe SwapProfile())
            Logger.Write("Switching to generated crafting profile.");
            NeoProfileManager.Load(pt.Path);

            if(TreeRoot.Current.ToString() != "Order Bot")
            {
                Logger.Write(LoggerType.Warning,
                    "Current botbase not set to Order Bot. Switch to Order Bot to start crafting!");
            }
        }
    }
}