﻿using System;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using ff14bot;
using ff14bot.BotBases;
using ff14bot.Enums;
using ff14bot.NeoProfiles;

namespace CraftBuddy
{
    public partial class CraftWindow : Form
    {
        private readonly CraftBuddy _cb;
        private readonly cbLogger _cbLogger;

        public CraftWindow(CraftBuddy cbInstance)
        {
            InitializeComponent();

            _cb = cbInstance;
            _cbLogger = cbInstance.Logger;

            lb_recipes.DisplayMember = "Name";
            lb_recipes.ValueMember = "Id";
            PopulateRecipeList();

            cmbox_profile_template.DisplayMember = "Name";
            PopulateTemplates();
        }

        /// <summary>
        /// Clears out old entries from the input fields of the form and loads available recipes.
        /// </summary>
        /// <returns></returns>
        private void PopulateRecipeList()
        {
            if(_cb.Recipes == null)
            {
                MessageBox.Show("No recipes loaded!");
                return;
            }

            if(!_cb.Recipes.Any())
            {
                MessageBox.Show("No Recipes in Recipe list!");
                return;
            }

            // Clear old entries from the listbox, id and name textboxes and searchbox.
            lb_recipes.Items.Clear();
            searchbox_value.Clear();

            // Add all valid recipes to the listbox.
            foreach(Recipe recipe in _cb.Recipes.Where(recipe => recipe.IsValid()))
            {
                lb_recipes.Items.Add(recipe);
            }

            // Generate a new autocompletion source
            AutoCompleteStringCollection strCollSearchBox = GenerateAutoCompletion();
            if(strCollSearchBox != null)
                searchbox_value.AutoCompleteCustomSource = strCollSearchBox;
        }

        /// <summary>
        /// Loads templates into the template combo box of the form.
        /// </summary>
        private void PopulateTemplates()
        {
            if(!_cb.ProfileTemplates.Any())
            {
                return;
            }

            // Remove old entries in the template dropdown
            if(cmbox_profile_template.Items.Count > 0)
            {
                cmbox_profile_template.Items.Clear();
            }

            // Add new entries
            cmbox_profile_template.Items.AddRange(_cb.ProfileTemplates.ToArray());
            // Set the first item in the dropdown as the selected item.
            if(cmbox_profile_template.Items.Count > 0)
                cmbox_profile_template.SelectedIndex = 0;

            // Set the first item in the dropdown to the settings file's default value.
            if(_cb.Settings != null)
            {
                if(!string.IsNullOrEmpty(_cb.Settings.DefaultTemplate))
                {
                    var index = cmbox_profile_template.FindString(_cb.Settings.DefaultTemplate);
                    if(index != ListBox.NoMatches)
                    {
                        cmbox_profile_template.SelectedIndex = index;
                    }
                }
            }
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Generates the autocompletion dropdown for the search bar.
        /// </summary>
        /// <returns>AutoCompleteStringCollection</returns>
        private AutoCompleteStringCollection GenerateAutoCompletion()
        {
            // Create a stringcollection for the searchbox, this allows us to search for an item later on.
            AutoCompleteStringCollection autoCompleteSearch = new AutoCompleteStringCollection();
            foreach(Recipe recipe in _cb.Recipes)
                autoCompleteSearch.Add(recipe.Name);

            return autoCompleteSearch;
        }

        private void btn_generate_Click(object sender, EventArgs e)
        {
            // TODO: Check for exceptions on file in and output
            Recipe recipe = lb_recipes.SelectedItem as Recipe;
            if(recipe == null)
            {
                _cbLogger.Write(LoggerType.Error, "Failed to obtain recipe from listbox.");
                return;
            }

            ProfileTemplate pt = cmbox_profile_template.SelectedItem as ProfileTemplate;
            _cb.generateProfile(recipe, pt);
        }

        private void searchbox_value_TextChanged(object sender, EventArgs e)
        {
            if(lb_recipes.Items.Count <= 0)
                return;

            string searchTerm = searchbox_value.Text;
            if(string.IsNullOrWhiteSpace(searchTerm))
                return;

            // Logic to select the similar listbox entry from the searchterm specified.
            foreach(Recipe recipe in _cb.Recipes.Where(recipe => recipe.IsValid()))
            {
                if(recipe.IsSimilar(searchTerm))
                {
                    lb_recipes.SetSelected(lb_recipes.FindStringExact(recipe.Name), true);
                    break;
                }
            }
        }

        private void CraftWindow_Load(object sender, EventArgs e)
        {
        }

        private void btn_refresh_recipes_Click(object sender, EventArgs e)
        {
            if(_cb.IsValidCraftClass(Core.Me.CurrentJob))
            {
                _cb.Recipes = _cb.GetRecipes();
                // Reload recipes from file.
                if(_cb.Recipes.Any())
                {
                    // Populate the new recipes in the form.
                    PopulateRecipeList();
                }
            }
            else
            {
                MessageBox.Show("Invalid Crafting class selected, please change your job to a crafting class!", "Error");
            }
        }

        private void cmbox_profile_template_SelectedIndexChanged(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Reloads templates from file and generates new ProfileTemplate objects. The new objects are added to the template combobox.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_reload_templates_Click(object sender, EventArgs e)
        {
            // Reload files from templates dir and create new ProfileTemplate objects.
            _cb.ReloadTemplates();
            PopulateTemplates();
        }
    }
}